; Hexacon version 1 - a wavetable based drone instrument
; Hexacon is based on six cubicon oscillators, six multi-shape LFOs, a reverb
; and some live or algorithmic control.
; This orchestra is set up to show the live MIDI performance version.
; See below and choose your MIDI and audio I/O.
<CsoundSynthesizer>
<CsOptions>
; Choose your realtime audio output. Remove the semicolon from the beginning
; of the line/choice you want.
;-+rtaudio=PortAudio ; All platforms
;-+rtaudio=alsa ; Linux: ALSA (Advanced Linux Sound Architecture)
-+rtaudio=jack ; Linux JACK Audio Connection Kit
;-+rtaudio=mme ; Windows MultiMedia Extension (the Windows standard?)
;-+rtaudio=CoreAudio ; Mac OS standard

; Choose your realtime MIDI module, remove the semicolon at the beginning of
; the line with your choice.
;-+rtmidi=PortMIDI ; All platforms
;-+rtmidi=alsa ; Linux (Advanced Linux Sound Architecture)
-+rtmidi=alsaseq ; Linux (ALSA sequencer)
;-+rtmidi=coremidi ; Mac OS (CoreMidi?)
;-+rtmidi=mme ; Windows MultiMedia Extension
;-+rtmidi=winmm ; Windows

; Choose your actual MIDI input device, this depends on the MIDI module above.
; You can choose one specific MIDI input, for most rtMIDI drivers this is a
; single number. For alsa this is something like
; hw:0
; Use amidi -l to show alsa port.
; Use aconnect -li to list alsaseq input ports
-M14 ; ALSA seq MIDI through on most systems

; Leave these options and go down to sr = to set your samplerate
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>
; Set your samplerate, most systems will accept 44100. Other popular values:
; 48000, 88200, 96000
sr	=	48000 ; 48000
; Go to the bottom of the file, if you want to checkout the "algorithmic"
; control instrument.
ksmps	=	1
nchnls	=	2
0dbfs	=	1

; Include m_cubicon opcode and wave cubes
#include "m_cubicon.udo"
giCubePos = ftgen(0, 0, -27, -23, "cube_pos.ftable")
giAngel1 = ftgen(0, 0, -3456, -23, "angel_cube1.ftable")
giAngel2 = ftgen(0, 0, -3456, -23, "angel_cube2.ftable")
giHarm = ftgen(0, 0, -3456, -23, "harmonic_cube1.ftable")
giAlgo = ftgen(0, 0, -3456, -23, "algo_cube1.ftable")
giMixed = ftgen(0, 0, -3456, -23, "mixed_cube1.ftable")
giVoice = ftgen(0, 0, -3456, -23, "voice_cube.ftable")
giPos = ftgen(0, 0, -27, -23, "cube_pos.ftable")

; Make sure no instrument will be triggered by MIDI events
massign 1, -1
massign 2, -1
massign 3, -1
massign 4, -1
massign 5, -1
massign 6, -1
massign 7, -1
massign 8, -1
massign 9, -1
massign 10, 01
massign 11, -1
massign 12, -1
massign 13, -1
massign 14, -1
massign 15, -1
massign 16, -1

; Run the six LFOs
instr LFO
	; Set up LFO wave shapes
	iSine = ftgenonce(0, 0, 4096, 10, 1)
	iSpline = ftgenonce(0, 0, 4096, -8, -1, 2048, 1, 2048, -1)
	iTri = ftgenonce(0, 0, 4096, -7, -1, 2048, 1, 2048, -1)
	iRampUp = ftgenonce(0, 0, 4096, -7, -1, 4094, 1, 2, -1)
	iSquare = ftgenonce(0, 0, 4096, -7, -1, 2, 1, 2046, 1, 2, -1, 2046, -1)
	iPuls33 = ftgenonce(0, 0, 4096, -7, -1, 2, 1, 1363, 1, 2, -1, 2729, -1)
	iPuls25 = ftgenonce(0, 0, 4096, -7, 1, 1023, 1, 2, -1, 3071, -1)
	iRampDown = ftgenonce(0, 0, 4096, -7, -1, 2, 1, 4094, -1)
	iDouble = ftgenonce(0, 0, 4096, 19, 1, .6, 0, 0, 1.5, 1, .5, 0)
	iNumShapes init 8 ; maximum shape index (number of shapes - 1)
	iTabLen = ftlen(iSine) -1

	; Create LFO control channels
	chn_k("m_hexacon.LFO1.Out", 3)
	chn_k("m_hexacon.LFO2.Out", 3)
	chn_k("m_hexacon.LFO3.Out", 3)
	chn_k("m_hexacon.LFO4.Out", 3)
	chn_k("m_hexacon.LFO5.Out", 3)
	chn_k("m_hexacon.LFO6.Out", 3)
	chn_k("m_hexacon.LFO1.Rate", 3)
	chn_k("m_hexacon.LFO2.Rate", 3)
	chn_k("m_hexacon.LFO3.Rate", 3)
	chn_k("m_hexacon.LFO4.Rate", 3)
	chn_k("m_hexacon.LFO5.Rate", 3)
	chn_k("m_hexacon.LFO6.Rate", 3)
	chn_k("m_hexacon.LFO1.Shape", 3)
	chn_k("m_hexacon.LFO2.Shape", 3)
	chn_k("m_hexacon.LFO3.Shape", 3)
	chn_k("m_hexacon.LFO4.Shape", 3)
	chn_k("m_hexacon.LFO5.Shape", 3)
	chn_k("m_hexacon.LFO6.Shape", 3)
	chn_k("m_hexacon.LFO1.Phase", 3)
	chn_k("m_hexacon.LFO2.Phase", 3)
	chn_k("m_hexacon.LFO3.Phase", 3)
	chn_k("m_hexacon.LFO4.Phase", 3)
	chn_k("m_hexacon.LFO5.Phase", 3)
	chn_k("m_hexacon.LFO6.Phase", 3)
	chn_k("m_hexacon.LFO1.Reset", 3)
	chn_k("m_hexacon.LFO2.Reset", 3)
	chn_k("m_hexacon.LFO3.Reset", 3)
	chn_k("m_hexacon.LFO4.Reset", 3)
	chn_k("m_hexacon.LFO5.Reset", 3)
	chn_k("m_hexacon.LFO6.Reset", 3)

	; Global LFO channels, to use when LFOs are set to unison
	chn_k("m_hexacon.LFOs.Unison", 3) ; 0 for free running, 1, for unison
	chn_k("m_hexacon.LFOs.Shape", 3)
	chn_k("m_hexacon.LFOs.Rate", 3)
	chn_k("m_hexacon.LFOs.Spread", 3) ; phase spread 0-1
	chn_k("m_hexacon.LFOs.Reset", 3)

	; Set up general LFO variables
	kRate[] init 6 ; LFO rates
	kPhase[] init 6 ; LFO phases
	kReset[] init 6

	; Set up table morphing variables
	kIndex[] init 6 ; table reading index
	kTab1[] init 6 ; first wave shape number
	kTab2[] init 6 ; second wave shape number
	kWeight[] init 6 ; mix weight between wave shapes

	; Set LFO parameters
	if (chnget:k("m_hexacon.LFOs.Unison") <.5) then
		kRate[0] = chnget:k("m_hexacon.LFO1.Rate")
		kRate[1] = chnget:k("m_hexacon.LFO2.Rate")
		kRate[2] = chnget:k("m_hexacon.LFO3.Rate")
		kRate[3] = chnget:k("m_hexacon.LFO4.Rate")
		kRate[4] = chnget:k("m_hexacon.LFO5.Rate")
		kRate[5] = chnget:k("m_hexacon.LFO6.Rate")
		kPhase[0] = chnget:k("m_hexacon.LFO1.Phase")
		kPhase[1] = chnget:k("m_hexacon.LFO2.Phase")
		kPhase[2] = chnget:k("m_hexacon.LFO3.Phase")
		kPhase[3] = chnget:k("m_hexacon.LFO4.Phase")
		kPhase[4] = chnget:k("m_hexacon.LFO5.Phase")
		kPhase[5] = chnget:k("m_hexacon.LFO6.Phase")
		kTab1[0] = wrap(int(chnget:k("m_hexacon.LFO1.Shape")), 0, iNumShapes)
		kTab1[1] = wrap(int(chnget:k("m_hexacon.LFO2.Shape")), 0, iNumShapes)
		kTab1[2] = wrap(int(chnget:k("m_hexacon.LFO3.Shape")), 0, iNumShapes)
		kTab1[3] = wrap(int(chnget:k("m_hexacon.LFO4.Shape")), 0, iNumShapes)
		kTab1[4] = wrap(int(chnget:k("m_hexacon.LFO5.Shape")), 0, iNumShapes)
		kTab1[5] = wrap(int(chnget:k("m_hexacon.LFO6.Shape")), 0, iNumShapes)
		kTab2[0] = wrap((kTab1[0] +1), 0, iNumShapes)
		kTab2[1] = wrap((kTab1[1] +1), 0, iNumShapes)
		kTab2[2] = wrap((kTab1[2] +1), 0, iNumShapes)
		kTab2[3] = wrap((kTab1[3] +1), 0, iNumShapes)
		kTab2[4] = wrap((kTab1[4] +1), 0, iNumShapes)
		kTab2[5] = wrap((kTab1[5] +1), 0, iNumShapes)
		kWeight[0] = frac(chnget:k("m_hexacon.LFO1.Shape"))
		kWeight[1] = frac(chnget:k("m_hexacon.LFO2.Shape"))
		kWeight[2] = frac(chnget:k("m_hexacon.LFO3.Shape"))
		kWeight[3] = frac(chnget:k("m_hexacon.LFO4.Shape"))
		kWeight[4] = frac(chnget:k("m_hexacon.LFO5.Shape"))
		kWeight[5] = frac(chnget:k("m_hexacon.LFO6.Shape"))
		kReset[0] = chnget:k("m_hexacon.LFO1.Reset")
		kReset[1] = chnget:k("m_hexacon.LFO2.Reset")
		kReset[2] = chnget:k("m_hexacon.LFO3.Reset")
		kReset[3] = chnget:k("m_hexacon.LFO4.Reset")
		kReset[4] = chnget:k("m_hexacon.LFO5.Reset")
		kReset[5] = chnget:k("m_hexacon.LFO6.Reset")
	else ; it's unison
		kRate[0] = chnget:k("m_hexacon.LFOs.Rate")
		kRate[1] = kRate[0]
		kRate[2] = kRate[0]
		kRate[3] = kRate[0]
		kRate[4] = kRate[0]
		kRate[5] = kRate[0]
		kPhase[0] = 0
		kPhase[1] = chnget:k("m_hexacon.LFOs.Spread") / 6
		kPhase[2] = kPhase[1] + kPhase[1]
		kPhase[3] = kPhase[2] + kPhase[1]
		kPhase[4] = kPhase[3] + kPhase[1]
		kPhase[5] = kPhase[4] + kPhase[1]
		kReset[0] = chnget:k("m_hexacon.LFOs.Reset")
		kReset[1] = kReset[0]
		kReset[2] = kReset[0]
		kReset[3] = kReset[0]
		kReset[4] = kReset[0]
		kReset[5] = kReset[0]
		kTab1[0] = wrap(int(chnget:k("m_hexacon.LFOs.Shape")), 0, iNumShapes)
		kTab1[1] = kTab1[0]
		kTab1[2] = kTab1[0]
		kTab1[3] = kTab1[0]
		kTab1[4] = kTab1[0]
		kTab1[5] = kTab1[0]
		kTab2[0] = wrap((kTab1[0] + 1), 0, iNumShapes)
		kTab2[1] = kTab2[0]
		kTab2[2] = kTab2[0]
		kTab2[3] = kTab2[0]
		kTab2[4] = kTab2[0]
		kTab2[5] = kTab2[0]
		kWeight[0] = frac(chnget:k("m_hexacon.LFOs.Shape"))
		kWeight[1] = kWeight[0]
		kWeight[2] = kWeight[0]
		kWeight[3] = kWeight[0]
		kWeight[4] = kWeight[0]
		kWeight[5] = kWeight[0]
	endif

	; Set up the table reading indices
	kIndex[0] = wrap(((trigphasor(kReset[0], (kRate[0] / kr), 0, 1) + kPhase[0]) * iTabLen), 0, iTabLen)
	kIndex[1] = wrap(((trigphasor(kReset[1], (kRate[1] / kr), 0, 1) + kPhase[1]) * iTabLen), 0, iTabLen)
	kIndex[2] = wrap(((trigphasor(kReset[2], (kRate[2] / kr), 0, 1) + kPhase[2]) * iTabLen), 0, iTabLen)
	kIndex[3] = wrap(((trigphasor(kReset[3], (kRate[3] / kr), 0, 1) + kPhase[3]) * iTabLen), 0, iTabLen)
	kIndex[4] = wrap(((trigphasor(kReset[4], (kRate[4] / kr), 0, 1) + kPhase[4]) * iTabLen), 0, iTabLen)
	kIndex[5] = wrap(((trigphasor(kReset[5], (kRate[5] / kr), 0, 1) + kPhase[5]) * iTabLen), 0, iTabLen)

	; Produce the actual LFOs
	chnset(tabmorph(kIndex[0], kWeight[0], kTab1[0], kTab2[0], iSine, iTri, iSpline, iRampUp, iSquare, iPuls33, iPuls25, iRampDown, iDouble), "m_hexacon.LFO1.Out")
	chnset(tabmorph(kIndex[1], kWeight[1], kTab1[1], kTab2[1], iSine, iTri, iSpline, iRampUp, iSquare, iPuls33, iPuls25, iRampDown, iDouble), "m_hexacon.LFO2.Out")
	chnset(tabmorph(kIndex[2], kWeight[2], kTab1[2], kTab2[2], iSine, iTri, iSpline, iRampUp, iSquare, iPuls33, iPuls25, iRampDown, iDouble), "m_hexacon.LFO3.Out")
	chnset(tabmorph(kIndex[3], kWeight[3], kTab1[3], kTab2[3], iSine, iTri, iSpline, iRampUp, iSquare, iPuls33, iPuls25, iRampDown, iDouble), "m_hexacon.LFO4.Out")
	chnset(tabmorph(kIndex[4], kWeight[4], kTab1[4], kTab2[4], iSine, iTri, iSpline, iRampUp, iSquare, iPuls33, iPuls25, iRampDown, iDouble), "m_hexacon.LFO5.Out")
	chnset(tabmorph(kIndex[5], kWeight[5], kTab1[5], kTab2[5], iSine, iTri, iSpline, iRampUp, iSquare, iPuls33, iPuls25, iRampDown, iDouble), "m_hexacon.LFO6.Out")

endin

; run the six m_cubicon wavetable oscillators
instr Oscillator
	; Set up wave cubes for each of the six oscillators
	iCube1 init giAngel1
	iCube2 init giAngel2
	iCube3 init giHarm
	iCube4 init giAlgo
	iCube5 init giMixed
	iCube6 init giVoice

	; Set default arguments for m_cubicon
	iConfTab init 0
	iQuality init 0

	; Create control channels
	chn_k("m_hexacon.Osc1.Pitch", 3)
	chn_k("m_hexacon.Osc2.Pitch", 3)
	chn_k("m_hexacon.Osc3.Pitch", 3)
	chn_k("m_hexacon.Osc4.Pitch", 3)
	chn_k("m_hexacon.Osc5.Pitch", 3)
	chn_k("m_hexacon.Osc6.Pitch", 3)
	chn_k("m_hexacon.Osc1.X", 3)
	chn_k("m_hexacon.Osc2.X", 3)
	chn_k("m_hexacon.Osc3.X", 3)
	chn_k("m_hexacon.Osc4.X", 3)
	chn_k("m_hexacon.Osc5.X", 3)
	chn_k("m_hexacon.Osc6.X", 3)
	chn_k("m_hexacon.Osc1.Y", 3)
	chn_k("m_hexacon.Osc2.Y", 3)
	chn_k("m_hexacon.Osc3.Y", 3)
	chn_k("m_hexacon.Osc4.Y", 3)
	chn_k("m_hexacon.Osc5.Y", 3)
	chn_k("m_hexacon.Osc6.Y", 3)
	chn_k("m_hexacon.Osc1.Z", 3)
	chn_k("m_hexacon.Osc2.Z", 3)
	chn_k("m_hexacon.Osc3.Z", 3)
	chn_k("m_hexacon.Osc4.Z", 3)
	chn_k("m_hexacon.Osc5.Z", 3)
	chn_k("m_hexacon.Osc6.Z", 3)
	chn_k("m_hexacon.Osc1.Vol", 3)
	chn_k("m_hexacon.Osc2.Vol", 3)
	chn_k("m_hexacon.Osc3.Vol", 3)
	chn_k("m_hexacon.Osc4.Vol", 3)
	chn_k("m_hexacon.Osc5.Vol", 3)
	chn_k("m_hexacon.Osc6.Vol", 3)
	chn_k("m_hexacon.Osc1.Pan", 3)
	chn_k("m_hexacon.Osc2.Pan", 3)
	chn_k("m_hexacon.Osc3.Pan", 3)
	chn_k("m_hexacon.Osc4.Pan", 3)
	chn_k("m_hexacon.Osc5.Pan", 3)
	chn_k("m_hexacon.Osc6.Pan", 3)

	; Global/unison controls
	chn_k("m_hexacon.Oscs.Unison", 3)
	chn_k("m_hexacon.Oscs.Pitch", 3)
	chn_k("m_hexacon.Oscs.PitchSpread", 3)
	chn_k("m_hexacon.Oscs.Vol", 3)
	chn_k("m_hexacon.Oscs.PanSpread", 3)
	chn_k("m_hexacon.Oscs.X", 3)
	chn_k("m_hexacon.Oscs.Y", 3)
	chn_k("m_hexacon.Oscs.Z", 3)
	chn_k("m_hexacon.Oscs.XSpread", 3)
	chn_k("m_hexacon.Oscs.YSpread", 3)
	chn_k("m_hexacon.Oscs.ZSpread", 3)
	chn_k("m_hexacon.Oscs.Shift", 3) ; chord shift
	chn_k("m_hexacon.Oscs.Porta", 3)

	; Stereo audio output channels
	chn_a("m_hexacon.Oscs.Left", 3)
	chn_a("m_hexacon.Oscs.Right", 3)

	; Set up internal oscillator variables
	aOsc[] init 6 ; individual oscillator output
	kPitch[] init 6
	kShift init 0 ; shift amount
	kShiftNotes init 0
	kShiftOctaves init 0
	kShiftIndex init 0
	kOctave[] = fillarray(1, 1, 1, 1, 1, 1) ; octave shift factors
	kSpread init 0
	kCurSpread init 0
	kSpreadStep init 0
	kSpreads[] = fillarray(1, 1, 1, 1, 1, 1)
		; pitch spread factors for each oscillator
	kPorta = chnget:k("m_hexacon.Oscs.Porta")
	kX[] init 6
	kY[] init 6
	kZ[] init 6
	kGlobalX init 0
	kGlobalY init 0
	kGlobalZ init 0
	kXOffset[] init 6
	kYOffset[] init 6
	kZOffset[] init 6
	kVol[] init 6
	kPan[] init 6

	; Calculate the actual oscillator parameters
	; Octave shift
	if (changed(chnget:k("m_hexacon.Oscs.Shift")) != 0) then ; a new shift value
		kShift = round(chnget:k("m_hexacon.Oscs.Shift"))
		kShiftOctaves = int(kShift / 6)
		kShiftNotes = kShift - (kShiftOctaves * 6)
		if (kShiftNotes <0) then
			kShiftNotes += 6
		else
			kShiftOctaves += 1
		endif
		kShiftIndex = 0
		while (kShiftIndex <6) do
			if (kShiftIndex == kShiftNotes) then
				kShiftOctaves -= 1
			endif
			kOctave[kShiftIndex] = 2^kShiftOctaves
			kShiftIndex += 1
		od
	endif

	; Set the other parameters
	if (chnget:k("m_hexacon.Oscs.Unison") <.5) then ; individual control
		; Calculate spread
		kSpread = limit(chnget:k("m_hexacon.Oscs.PitchSpread"), 0, 1)
		if (changed(kSpread) != 0) then
			kSpreadStep = kSpread * 100 / 3
			kCurSpread = kSpread * 100
			kShiftIndex = 0
			while (kShiftIndex <6) do
				kSpreads[kShiftIndex] = cent(kCurSpread)
				kCurSpread -= kSpreadStep
				kShiftIndex += 1
			od
		endif ; End of spread calculation

		; Calculate pitches
		kPitch[0] = portk((chnget:k("m_hexacon.Osc1.Pitch") * kOctave[0] * kSpreads[0]), kPorta)
		kPitch[1] = portk((chnget:k("m_hexacon.Osc2.Pitch") * kOctave[1] * kSpreads[1]), kPorta)
		kPitch[2] = portk((chnget:k("m_hexacon.Osc3.Pitch") * kOctave[2] * kSpreads[2]), kPorta)
		kPitch[3] = portk((chnget:k("m_hexacon.Osc4.Pitch") * kOctave[3] * kSpreads[3]), kPorta)
		kPitch[4] = portk((chnget:k("m_hexacon.Osc5.Pitch") * kOctave[4] * kSpreads[4]), kPorta)
		kPitch[5] = portk((chnget:k("m_hexacon.Osc6.Pitch") * kOctave[5] * kSpreads[5]), kPorta)

		; Set X, Y and Z wavetable positions
		kX[0] = chnget:k("m_hexacon.Osc1.X")
		kX[1] = chnget:k("m_hexacon.Osc2.X")
		kX[2] = chnget:k("m_hexacon.Osc3.X")
		kX[3] = chnget:k("m_hexacon.Osc4.X")
		kX[4] = chnget:k("m_hexacon.Osc5.X")
		kX[5] = chnget:k("m_hexacon.Osc6.X")
		kY[0] = chnget:k("m_hexacon.Osc1.Y")
		kY[1] = chnget:k("m_hexacon.Osc2.Y")
		kY[2] = chnget:k("m_hexacon.Osc3.Y")
		kY[3] = chnget:k("m_hexacon.Osc4.Y")
		kY[4] = chnget:k("m_hexacon.Osc5.Y")
		kY[5] = chnget:k("m_hexacon.Osc6.Y")
		kZ[0] = chnget:k("m_hexacon.Osc1.Z")
		kZ[1] = chnget:k("m_hexacon.Osc2.Z")
		kZ[2] = chnget:k("m_hexacon.Osc3.Z")
		kZ[3] = chnget:k("m_hexacon.Osc4.Z")
		kZ[4] = chnget:k("m_hexacon.Osc5.Z")
		kZ[5] = chnget:k("m_hexacon.Osc6.Z")

		; Set volume and pan
		kVol[0] = chnget:k("m_hexacon.Osc1.Vol")
		kVol[1] = chnget:k("m_hexacon.Osc2.Vol")
		kVol[2] = chnget:k("m_hexacon.Osc3.Vol")
		kVol[3] = chnget:k("m_hexacon.Osc4.Vol")
		kVol[4] = chnget:k("m_hexacon.Osc5.Vol")
		kVol[5] = chnget:k("m_hexacon.Osc6.Vol")
		kPan[0] = portk(chnget:k("m_hexacon.Osc1.Pan"), kPorta)
		kPan[1] = portk(chnget:k("m_hexacon.Osc2.Pan"), kPorta)
		kPan[2] = portk(chnget:k("m_hexacon.Osc3.Pan"), kPorta)
		kPan[3] = portk(chnget:k("m_hexacon.Osc4.Pan"), kPorta)
		kPan[4] = portk(chnget:k("m_hexacon.Osc5.Pan"), kPorta)
		kPan[5] = portk(chnget:k("m_hexacon.Osc6.Pan"), kPorta)
	else ; unison control
		; Calculate spread
		kSpread = chnget:k("m_hexacon.Oscs.PitchSpread")
		if (changed(kSpread) != 0) then
			if ((kSpread <2) && (kSpread >-2)) then ; detune in cents
				kTmpSpread = limit(kSpread, -1, 1)
				kSpreadStep = kTmpSpread * 100 / 3
				kCurSpread = kTmpSpread
				kShiftIndex = 0
				while (kShiftIndex <6) do
					kSpreads[kShiftIndex] = cent(kCurSpread)
					kCurSpread -= kSpreadStep
					kShiftIndex += 1
				od
			else ; detune in semitones
				kSpreadStep = round(limit(kSpread, -13, 13))
				if (kSpreadStep <0) then
					kSpreadStep += 1
				else
					kSpreadStep -= 1
				endif
				kCurSpread = kSpreadStep * 3
				kShiftIndex = 0
				while (kShiftIndex <6) do
					kSpreads[kShiftIndex] = semitone(kCurSpread)
					kCurSpread -= kSpreadStep
					kShiftIndex += 1
				od
			endif
		endif ; End of spread calculation

		; Calculate pitches
		kTmpPitch = chnget:k("m_hexacon.Oscs.Pitch")
		kPitch[0] = portk((kTmpPitch * kOctave[0] * kSpreads[0]), kPorta)
		kPitch[1] = portk((kTmpPitch * kOctave[1] * kSpreads[1]), kPorta)
		kPitch[2] = portk((kTmpPitch * kOctave[2] * kSpreads[2]), kPorta)
		kPitch[3] = portk((kTmpPitch * kOctave[3] * kSpreads[3]), kPorta)
		kPitch[4] = portk((kTmpPitch * kOctave[4] * kSpreads[4]), kPorta)
		kPitch[5] = portk((kTmpPitch * kOctave[5] * kSpreads[5]), kPorta)

		; Calculate X, Y and Z wavetable positions
		kGlobalX = chnget:k("m_hexacon.Oscs.X")
		kGlobalY = chnget:k("m_hexacon.Oscs.Y")
		kGlobalZ = chnget:k("m_hexacon.Oscs.Z")
		kCurX = chnget:k("m_hexacon.Oscs.XSpread")
		kCurY = chnget:k("m_hexacon.Oscs.YSpread")
		kCurZ = chnget:k("m_hexacon.Oscs.ZSpread")
		kXStep = kCurX / 6
		kYStep = kCurY / 6
		kZStep = kCurZ / 6
		kShiftIndex = 5
		while (kShiftIndex >=0) do
			kX[kShiftIndex] = kGlobalX + kCurX
			kY[kShiftIndex] = kGlobalY + kCurY
			kZ[kShiftIndex] = kGlobalZ + kCurZ
			kCurX -= kXStep
			kCurY -= kYStep
			kCurZ -= kYStep
			kShiftIndex -= 1
		od

		; Set volume
		kVol[0] = chnget:k("m_hexacon.Oscs.Vol")
		kVol[1] = kVol[0]
		kVol[2] = kVol[0]
		kVol[3] = kVol[0]
		kVol[4] = kVol[0]
		kVol[5] = kVol[0]

		; Set pan positions
		kPanSpread = limit(chnget:k("m_hexacon.Oscs.PanSpread"), 0, 1)
		kPanStep = kPanSpread / 6
		kCurPan = .5 - (3 * kPanStep)
		kShiftIndex = 0
		while (kShiftIndex <6) do
			kPan[kShiftIndex] = kCurPan
			kCurPan += kPanStep
			kShiftIndex += 1
		od
	endif ; End of oscillator parameter calcuation

	; Produce oscillator output
	aOsc[0] = m_cubicon(kPitch[0], kX[0], kY[0], kZ[0], 1, iCube1, giPos, iConfTab, iQuality)
	aOsc[1] = m_cubicon(kPitch[1], kX[1], kY[1], kZ[1], 1, iCube1, giPos, iConfTab, iQuality)
	aOsc[2] = m_cubicon(kPitch[2], kX[2], kY[2], kZ[2], 1, iCube1, giPos, iConfTab, iQuality)
	aOsc[3] = m_cubicon(kPitch[3], kX[3], kY[3], kZ[3], 1, iCube1, giPos, iConfTab, iQuality)
	aOsc[4] = m_cubicon(kPitch[4], kX[4], kY[4], kZ[4], 1, iCube1, giPos, iConfTab, iQuality)
	aOsc[5] = m_cubicon(kPitch[5], kX[5], kY[5], kZ[5], 1, iCube1, giPos, iConfTab, iQuality)

	; Pan the oscillators
	aLeft init 0
	aRight init 0
	aLefts[] init 6
	aRights[] init 6
	aLefts[0], aRights[0] pan2 (aOsc[0] * kVol[0]), kPan[0]
	aLefts[1], aRights[1] pan2 (aOsc[1] * kVol[1]), kPan[1]
	aLefts[2], aRights[2] pan2 (aOsc[2] * kVol[2]), kPan[2]
	aLefts[3], aRights[3] pan2 (aOsc[3] * kVol[3]), kPan[3]
	aLefts[4], aRights[4] pan2 (aOsc[4] * kVol[4]), kPan[4]
	aLefts[5], aRights[5] pan2 (aOsc[5] * kVol[5]), kPan[5]
	aLeft = (aLefts[0] + aLefts[1] + aLefts[2] + aLefts[3] + aLefts[4] \
		+ aLefts[5])
	aRight = (aRights[0] + aRights[1] + aRights[2] + aRights[3] + aRights[4] \
		+ aRights[5])
	chnset(aLeft, "m_hexacon.Oscs.Left")
	chnset(aRight, "m_hexacon.Oscs.Right")
endin

; Global reverb
instr Reverb
	; Volume fade-in
	kVol = linseg(0, 1, 1, (p3 - 1), 1)
	; Set up reverb parameters
	kDry init .5
	kFb init .95
	kCutoff init 5000
	iPitchMod init 1

	; Audio input from oscillator stereo mix
	aInLeft = chnget:a("m_hexacon.Oscs.Left") * kVol
	aInRight = chnget:a("m_hexacon.Oscs.Right") * kVol
	aRevLeft, aRevRight reverbsc aInLeft, aInRight, kFb, kCutoff, sr, iPitchMod
	aOutLeft = aInLeft * kDry + aRevLeft * (1 - kDry)
	aOutRight = aInRight * kDry + aRevRight * (1 - kDry)
	outs(aOutLeft, aOutRight)
endin

; Demo ontrol instrument
; This instrument creates hardcoded control signals (k-rate variables) and
; sends them directly to the respective control channels.
instr Control
	kFade = linseg(0, .05, 1, (p3 - .05), 1)
	kShift = linseg(0, p3*.8, 0)
	kSpread = linseg(0, p3*.25, .25, p3*.25, .25, p3*.25, 8, p3*.25, 8)
	kYSpread = linseg(0, p3/4, 0, p3/4, 1, p3/4, 2, p3/4, 3)
	kShape = linseg(0, p3*.9, 1, p3*.1, 1)
	chnset(1*kFade, "m_hexacon.LFOs.Unison")
	chnset(.3*kFade, "m_hexacon.LFOs.Rate")
	chnset(kShape, "m_hexacon.LFOs.Shape")
	chnset(1*kFade, "m_hexacon.LFOs.Spread")
	chnset(1, "m_hexacon.Oscs.Unison")
	chnset(220, "m_hexacon.Oscs.Pitch")
	chnset(.5*kFade, "m_hexacon.Oscs.Vol")
	chnset(.0001, "m_hexacon.Oscs.Porta")
	chnset(kShift, "m_hexacon.Oscs.Shift")
	chnset(1*kFade, "m_hexacon.Oscs.PanSpread")
	chnset(kSpread, "m_hexacon.Oscs.PitchSpread")
	chnset((chnget:k("m_hexacon.LFO1.Out") * 1.5) +1, "m_hexacon.Oscs.X")
	chnset((chnget:k("m_hexacon.LFO2.Out") * 1.5) + 1, "m_hexacon.Oscs.Z")
	chnset((chnget:k("m_hexacon.LFO3.Out") * 1.5) + 1, "m_hexacon.Oscs.Y")
	chnset(kYSpread, "m_hexacon.Oscs.XSpread")
	chnset((3 - kYSpread), "m_hexacon.Oscs.ZSpread")
endin

; MIDI control instrument
; This instrument uses MIDI notes to control the pitch of the oscillators
instr Keyboard
	; Create the MIDI input variables
	kStatus init 0 ; mIDI message byte
	kChannel init 0 ; MIDI channel, ignored, this instrument will accept
		; input from all channels
	kByte1 init 60 ; first MIDI message byte
	kByte2 init 127 ; second MIDI message byte
	kHeldNotes init 0 ; actual number of keys held down
	kNumNotes init 0 ; number of notes added to the current chord
	kNotePos init 0 ; note position in the input
	kNoteIndex init 0 ; used in distribution logic
	kShift init 0 ; Chord/note shift
	iC init cpsmidinn(60) ; pitch of middle C for initialisation
	iMaxVol init .7

	; Arrays for the notes
	kInputs[] init 6 ; Input array with combined note and volume values
	kInputsBuffer[] init 6 ; distributed notes
	kSortedInputs[] init 6 ; correctly distributed and sorted buffer
	kPitches[] = fillarray(iC, iC, iC, iC, iC, iC)
	kMasterVol init .5 ; Master volume
	kVols[] = fillarray(.3, .3, .3, .3, .3, .3)
	kPosDepth init 0 ; modulation depth of X, Y, Z modulation
	kXs[] init 6 ; X wavetable positions
	kYs[] init 6 ; Y wavetable positions
	kZs[] init 6 ; Z wavetable positions

	; Get MIDI input
	kStatus, kChannel, kByte1, kByte2 midiin

	; Process MIDI input
	if (kStatus == 144) then ; note on
		; Increase number of held notes and add MIDI note + velocity to input
		if (kHeldNotes == 0) then
			kNumNotes = 0
		endif
		kHeldNotes += 1
		kNotePos = wrap(kNumNotes, 0, 5)
		kNumNotes += 1
		kInputs[kNotePos] = kByte1 + (kByte2 / 256)
		
		; Distribute input notes across output array
		if (kNumNotes <=6) then
			kStep = 6 / kNumNotes
			kNoteIndex = 0
			kInputIndex = 0
			kNext = kStep -1 ; because indices count from 0
			while (kNoteIndex <6) do
				if (kNoteIndex > round(kNext)) then
					kInputIndex += 1
					kNext += kStep
				endif
				kInputsBuffer[kNoteIndex] = kInputs[kInputIndex]
				kNoteIndex += 1
			od
		else ; more than six notes, just copy the input array
			kInputsBuffer = kInputs
		endif ; End of note distribution
		
		; Sort notes
		kSortedInputs = sorta(kInputsBuffer)
		kPitches[1] = cpsmidinn(int(kSortedInputs[1]))
		kPitches[2] = cpsmidinn(int(kSortedInputs[2]))
		kPitches[3] = cpsmidinn(int(kSortedInputs[3]))
		kPitches[4] = cpsmidinn(int(kSortedInputs[4]))
		kPitches[5] = cpsmidinn(int(kSortedInputs[5]))
		kVols[0] = linlin(frac(kSortedInputs[0]), .05, iMaxVol, 0, 1)
		kVols[1] = linlin(frac(kSortedInputs[1]), .05, iMaxVol, 0, 1)
		kVols[2] = linlin(frac(kSortedInputs[2]), .05, iMaxVol, 0, 1)
		kVols[3] = linlin(frac(kSortedInputs[3]), .05, iMaxVol, 0, 1)
		kVols[4] = linlin(frac(kSortedInputs[4]), .05, iMaxVol, 0, 1)
		kVols[5] = linlin(frac(kSortedInputs[5]), .05, iMaxVol, 0, 1)
	elseif (kStatus == 128) then ; note off, decrease number of held notes
		kHeldNotes -= 1
		if (kHeldNotes <0) then
			kHeldNotes = 0
		endif
	elseif (kStatus == 176) then ; MIDI CC
		if (kByte1 == 1) then ; modwheel for kPosDepth
			kPosDepth = linlin(kByte2, 0, 1, 0, 127)
		elseif (kByte1 == 7) then ; volume
			kMasterVol = linlin(kByte2, .05, iMaxVol, 0, 127)
		elseif (kByte1 == 11) then ; shift
			kShift = linlin(kByte2, -6, 6, 0, 127)
		endif
	endif

	; Below LFOs will be set up and connected to the oscillators X, Y and Z
	; inputs and thepitch and volume information is sent through the channels.

	; Set LFO parameters
	chnset(1, "m_hexacon.LFOs.Unison") ; unison mode
	chnset(1, "m_hexacon.LFOs.Spread") ; full phase spread
	chnset(0, "m_hexacon.LFOs.Shape") ; sine LFOs
	chnset(.5, "m_hexacon.LFOs.Rate") ; rate of 1/2 Hz

	; Set static oscillator parameters
	chnset(.05, "m_hexacon.Oscs.Porta")
	chnset(0, "m_hexacon.Osc1.Pan")
	chnset(.2, "m_hexacon.Osc2.Pan")
	chnset(.4, "m_hexacon.Osc3.Pan")
	chnset(.6, "m_hexacon.Osc4.Pan")
	chnset(.8, "m_hexacon.Osc5.Pan")
	chnset(1, "m_hexacon.Osc6.Pan")
	chnset(0, "m_hexacon.Oscs.Unison")
	chnset(.2, "m_hexacon.Oscs.PitchSpread")
	chnset(kShift, "m_hexacon.Oscs.Shift")

	; Create X, Y and Z modulation using the LFOs
	kXs[0] = linlin(chnget:k("m_hexacon.LFO1.Out"), 1, 4, -1, 1) * kPosDepth
	kXs[1] = linlin(chnget:k("m_hexacon.LFO2.Out"), 2, 5, -1, 1) * kPosDepth
	kXs[2] = linlin(chnget:k("m_hexacon.LFO3.Out"), 0, 3, -1, 1) * kPosDepth
	kXs[3] = linlin(chnget:k("m_hexacon.LFO4.Out"), 1, 4, -1, 1) * kPosDepth
	kXs[4] = linlin(chnget:k("m_hexacon.LFO5.Out"), 2, 5, -1, 1) * kPosDepth
	kXs[5] = linlin(chnget:k("m_hexacon.LFO6.Out"), 0, 3, -1, 1) * kPosDepth
	kYs[0] = linlin(chnget:k("m_hexacon.LFO1.Out"), 1, 4, -1, 1) * kPosDepth
	kYs[1] = linlin(chnget:k("m_hexacon.LFO2.Out"), 2, 5, -1, 1) * kPosDepth
	kYs[2] = linlin(chnget:k("m_hexacon.LFO3.Out"), 0, 3, -1, 1) * kPosDepth
	kYs[3] = linlin(chnget:k("m_hexacon.LFO4.Out"), 1, 4, -1, 1) * kPosDepth
	kYs[4] = linlin(chnget:k("m_hexacon.LFO5.Out"), 2, 5, -1, 1) * kPosDepth
	kYs[5] = linlin(chnget:k("m_hexacon.LFO6.Out"), 0, 3, -1, 1) * kPosDepth
	kZs[0] = linlin(chnget:k("m_hexacon.LFO1.Out"), 3, 0, -1, 1) * kPosDepth
	kZs[1] = linlin(chnget:k("m_hexacon.LFO2.Out"), 4, 1, -1, 1) * kPosDepth
	kZs[2] = linlin(chnget:k("m_hexacon.LFO3.Out"), 5, 2, -1, 1) * kPosDepth
	kZs[3] = linlin(chnget:k("m_hexacon.LFO4.Out"), 3, 0, -1, 1) * kPosDepth
	kZs[4] = linlin(chnget:k("m_hexacon.LFO5.Out"), 4, 1, -1, 1) * kPosDepth
	kZs[5] = linlin(chnget:k("m_hexacon.LFO6.Out"), 5, 2, -1, 1) * kPosDepth

	; Send wavetable position
	chnset(kXs[0], "m_hexacon.Osc1.X")
	chnset(kXs[1], "m_hexacon.Osc2.X")
	chnset(kXs[2], "m_hexacon.Osc3.X")
	chnset(kXs[3], "m_hexacon.Osc4.X")
	chnset(kXs[4], "m_hexacon.Osc5.X")
	chnset(kXs[5], "m_hexacon.Osc6.X")
	chnset(kYs[0], "m_hexacon.Osc1.Y")
	chnset(kYs[1], "m_hexacon.Osc2.Y")
	chnset(kYs[2], "m_hexacon.Osc3.Y")
	chnset(kYs[3], "m_hexacon.Osc4.Y")
	chnset(kYs[4], "m_hexacon.Osc5.Y")
	chnset(kYs[5], "m_hexacon.Osc6.Y")
	chnset(kZs[0], "m_hexacon.Osc1.Z")
	chnset(kYs[1], "m_hexacon.Osc2.Y")
	chnset(kYs[2], "m_hexacon.Osc3.Y")
	chnset(kYs[3], "m_hexacon.Osc4.Y")
	chnset(kYs[4], "m_hexacon.Osc5.Y")
	chnset(kYs[5], "m_hexacon.Osc6.Y")

	; Send oscillator pitch and volume
	chnset(kPitches[0], "m_hexacon.Osc1.Pitch")
	chnset(kPitches[1], "m_hexacon.Osc2.Pitch")
	chnset(kPitches[2], "m_hexacon.Osc3.Pitch")
	chnset(kPitches[3], "m_hexacon.Osc4.Pitch")
	chnset(kPitches[4], "m_hexacon.Osc5.Pitch")
	chnset(kPitches[5], "m_hexacon.Osc6.Pitch")
	chnset(kVols[0] * kMasterVol, "m_hexacon.Osc1.Vol")
	chnset(kVols[1] * kMasterVol, "m_hexacon.Osc2.Vol")
	chnset(kVols[2] * kMasterVol, "m_hexacon.Osc3.Vol")
	chnset(kVols[3] * kMasterVol, "m_hexacon.Osc4.Vol")
	chnset(kVols[4] * kMasterVol, "m_hexacon.Osc5.Vol")
	chnset(kVols[5] * kMasterVol, "m_hexacon.Osc6.Vol")
endin

</CsInstruments>
; ==============================================
<CsScore>
; The line below starts the live MIDI instrument called Keyboard
; Put a semicolon in front of that line, if you don't want the MIDI control
i"Keyboard" 0 z

; The line below starts the purely Csound internal control instrument, using
; hardcoded k-rate signals
; Remove the semicolon of that line to run this instrument.
;i"Control" .01 35 ; This only affects things for 35 seconds

; Run the LFOs, oscillators and reverb
i"LFO" 0 z
i"Oscillator" 0 z
i"Reverb" 0 z
e
</CsScore>
</CsoundSynthesizer>
